/*
 * Copyright (c) 2013, The University of Oxford
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. Neither the name of the University of Oxford nor the names of its
 *    contributors may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef OSKAR_DFTW_M2M_3D_INDEXED_INPUT_CUDA_H_
#define OSKAR_DFTW_M2M_3D_INDEXED_INPUT_CUDA_H_

/**
 * @file oskar_dftw_m2m_3d_indexed_input_cuda.h
 */

#include <oskar_global.h>
#include <utility/oskar_vector_types.h>

#ifdef __cplusplus
extern "C" {
#endif

/**
 * @brief
 * CUDA function to perform a 3D complex-matrix-to-complex-matrix
 * single-precision DFT using supplied weights.
 *
 * @details
 * This CUDA function performs a 3D complex-matrix-to-complex-matrix DFT using
 * the supplied complex weights and complex input data, which is accessed
 * indirectly using supplied indices.
 *
 * The wavelength used to compute the supplied wavenumber must be in the
 * same units as the input positions.
 *
 * The input data must be supplied in an array of size \p n_out * \p n_types_in,
 * where \p n_types_in is the number of different types of input data. The
 * supplied array of indices determine which element of the input data array
 * is used for each input point. The input data array is accessed in such
 * a way that the output dimension must be the fastest varying.
 *
 * The computed points are returned in the \p d_output array, which must be
 * pre-sized to length n_out. The values in the \p d_output array are
 * the complex values for each output position.
 *
 * Note that all pointers refer to device memory.
 *
 * @param[in] n_in         Number of input points.
 * @param[in] wavenumber   Wavenumber (2 pi / wavelength).
 * @param[in] d_x_in       Array of input x positions.
 * @param[in] d_y_in       Array of input y positions.
 * @param[in] d_z_in       Array of input z positions.
 * @param[in] d_weights_in Array of complex DFT weights.
 * @param[in] n_out        Number of output points.
 * @param[in] d_x_out      Array of output 1/x positions.
 * @param[in] d_y_out      Array of output 1/y positions.
 * @param[in] d_z_out      Array of output 1/z positions.
 * @param[in] d_index_in   Index into input data array for each input.
 * @param[in] d_data       Array of complex input data (size n_out * n_types_in).
 * @param[out] d_output    Array of computed output points (see note, above).
 */
OSKAR_EXPORT
void oskar_dftw_m2m_3d_indexed_input_cuda_f(int n_in, float wavenumber,
        const float* d_x_in, const float* d_y_in, const float* d_z_in,
        const float2* d_weights_in, int n_out, const float* d_x_out,
        const float* d_y_out, const float* d_z_out, const int* d_index_in,
        const float4c* d_data, float4c* d_output);

/**
 * @brief
 * CUDA function to perform a 3D complex-matrix-to-complex-matrix
 * double-precision DFT using supplied weights.
 *
 * @details
 * This CUDA function performs a 3D complex-matrix-to-complex-matrix DFT using
 * the supplied complex weights and complex input data, which is accessed
 * indirectly using supplied indices.
 *
 * The wavelength used to compute the supplied wavenumber must be in the
 * same units as the input positions.
 *
 * The input data must be supplied in an array of size \p n_out * \p n_types_in,
 * where \p n_types_in is the number of different types of input data. The
 * supplied array of indices determine which element of the input data array
 * is used for each input point. The input data array is accessed in such
 * a way that the output dimension must be the fastest varying.
 *
 * The computed points are returned in the \p d_output array, which must be
 * pre-sized to length n_out. The values in the \p d_output array are
 * the complex values for each output position.
 *
 * Note that all pointers refer to device memory.
 *
 * @param[in] n_in         Number of input points.
 * @param[in] wavenumber   Wavenumber (2 pi / wavelength).
 * @param[in] d_x_in       Array of input x positions.
 * @param[in] d_y_in       Array of input y positions.
 * @param[in] d_z_in       Array of input z positions.
 * @param[in] d_weights_in Array of complex DFT weights.
 * @param[in] n_out        Number of output points.
 * @param[in] d_x_out      Array of output 1/x positions.
 * @param[in] d_y_out      Array of output 1/y positions.
 * @param[in] d_z_out      Array of output 1/z positions.
 * @param[in] d_index_in   Index into input data array for each input.
 * @param[in] d_data       Array of complex input data (size n_out * n_types_in).
 * @param[out] d_output    Array of computed output points (see note, above).
 */
OSKAR_EXPORT
void oskar_dftw_m2m_3d_indexed_input_cuda_d(int n_in, double wavenumber,
        const double* d_x_in, const double* d_y_in, const double* d_z_in,
        const double2* d_weights_in, int n_out, const double* d_x_out,
        const double* d_y_out, const double* d_z_out, const int* d_index_in,
        const double4c* d_data, double4c* d_output);

#ifdef __CUDACC__

/**
 * @brief
 * CUDA kernel to perform a 3D complex-matrix-to-complex-matrix
 * single-precision DFT using supplied weights.
 *
 * @details
 * This CUDA kernel performs a 3D complex-matrix-to-complex-matrix DFT using
 * the supplied complex weights and complex input data, which is accessed
 * indirectly using supplied indices.
 *
 * The wavelength used to compute the supplied wavenumber must be in the
 * same units as the input positions.
 *
 * Each thread evaluates a single output point, looping over all the input
 * points while performing a complex multiply-accumulate with the supplied
 * input DFT weights.
 *
 * The input data must be supplied in an array of size \p n_out * \p n_types_in,
 * where \p n_types_in is the number of different types of input data. The
 * supplied array of indices determine which element of the input data array
 * is used for each input point. The input data array is accessed in such
 * a way that the output dimension must be the fastest varying.
 *
 * The computed points are returned in the \p output array, which must be
 * pre-sized to length n_out. The values in the \p output array are
 * the complex values for each output position.
 *
 * @param[in] n_in         Number of input points.
 * @param[in] wavenumber   Wavenumber (2 pi / wavelength).
 * @param[in] x_in         Array of input x positions.
 * @param[in] y_in         Array of input y positions.
 * @param[in] z_in         Array of input z positions.
 * @param[in] weights_in   Array of complex DFT weights.
 * @param[in] n_out        Number of output points.
 * @param[in] x_out        Array of output 1/x positions.
 * @param[in] y_out        Array of output 1/y positions.
 * @param[in] z_out        Array of output 1/z positions.
 * @param[in] max_in_chunk Maximum input points per chunk.
 * @param[in] index_in     Index into input data array for each input.
 * @param[in] data         Array of complex input data (size n_out * n_types_in).
 * @param[out] output      Array of computed output points (see note, above).
 */
__global__
void oskar_dftw_m2m_3d_indexed_input_cudak_f(const int n_in,
        const float wavenumber,
        const float* __restrict__ x_in,
        const float* __restrict__ y_in,
        const float* __restrict__ z_in,
        const float2* __restrict__ weights_in,
        const int n_out,
        const float* __restrict__ x_out,
        const float* __restrict__ y_out,
        const float* __restrict__ z_out,
        const int max_in_chunk,
        const int* __restrict__ index_in,
        const float4c* __restrict__ data,
        float4c* __restrict__ output);

/**
 * @brief
 * CUDA kernel to perform a 3D complex-matrix-to-complex-matrix
 * double-precision DFT using supplied weights.
 *
 * @details
 * This CUDA kernel performs a 3D complex-matrix-to-complex-matrix DFT using
 * the supplied complex weights and complex input data, which is accessed
 * indirectly using supplied indices.
 *
 * The wavelength used to compute the supplied wavenumber must be in the
 * same units as the input positions.
 *
 * Each thread evaluates a single output point, looping over all the input
 * points while performing a complex multiply-accumulate with the supplied
 * input DFT weights.
 *
 * The input data must be supplied in an array of size \p n_out * \p n_types_in,
 * where \p n_types_in is the number of different types of input data. The
 * supplied array of indices determine which element of the input data array
 * is used for each input point. The input data array is accessed in such
 * a way that the output dimension must be the fastest varying.
 *
 * The computed points are returned in the \p output array, which must be
 * pre-sized to length n_out. The values in the \p output array are
 * the complex values for each output position.
 *
 * @param[in] n_in         Number of input points.
 * @param[in] wavenumber   Wavenumber (2 pi / wavelength).
 * @param[in] x_in         Array of input x positions.
 * @param[in] y_in         Array of input y positions.
 * @param[in] z_in         Array of input z positions.
 * @param[in] weights_in   Array of complex DFT weights.
 * @param[in] n_out        Number of output points.
 * @param[in] x_out        Array of output 1/x positions.
 * @param[in] y_out        Array of output 1/y positions.
 * @param[in] z_out        Array of output 1/z positions.
 * @param[in] max_in_chunk Maximum input points per chunk.
 * @param[in] index_in     Index into input data array for each input.
 * @param[in] data         Array of complex input data (size n_out * n_types_in).
 * @param[out] output      Array of computed output points (see note, above).
 */
__global__
void oskar_dftw_m2m_3d_indexed_input_cudak_d(const int n_in,
        const double wavenumber,
        const double* __restrict__ x_in,
        const double* __restrict__ y_in,
        const double* __restrict__ z_in,
        const double2* __restrict__ weights_in,
        const int n_out,
        const double* __restrict__ x_out,
        const double* __restrict__ y_out,
        const double* __restrict__ z_out,
        const int max_in_chunk,
        const int* __restrict__ index_in,
        const double4c* __restrict__ data,
        double4c* __restrict__ output);

#endif /* __CUDACC__ */

#ifdef __cplusplus
}
#endif

#endif /* OSKAR_DFTW_M2M_3D_INDEXED_INPUT_CUDA_H_ */
